/* 
 * noIdle v0.5 (date: 24-Feb-2002)
 *
 * Program that don't let user get more than a specific idle time.
 * 
 * (C) 2002 - Jos Martins (jemart@student.dei.uc.pt)
 * 
 * */

#include "define.h"

static int pipe_fd[2];

int main(void){
  
  int child, status=0,select_val;
  ssize_t count;
  char str[MAX_LINE]="", user[MAX_LINE]="jemart";
  struct timeval timeout,rtimeout;
  fd_set fdset;
  
  if(assignSignals()==EXIT_FAILURE){
    printf("error assigning signals...\n");
    return EXIT_FAILURE;
  };
  
  /* opens pipes */
  if( pipe(pipe_fd)==-1){
    perror("Error making pipe");
    return EXIT_FAILURE;
  };
  
  for(;;){
  printf(" cicle...\n");

  /* timeout of 28min=168sec */
  timeout.tv_sec=CICLE_TIME;
  timeout.tv_usec=0;
  /* read timeout of 10sec */
  rtimeout.tv_sec=5;
  rtimeout.tv_usec=0;
  
  /* Creates the processCmd information */
  child=fork();
  if(child==-1){
    /* error */
    perror("Error creating child");
    return EXIT_FAILURE;
  }else if(child==0){
    /* child */
    return processCmd(user);
  };
  /* father */
  
  /* receives information from pipe_fd[0] */
  for(;;){
    (void)memset(str,(int)'\0',MAX_LINE);
    
    /* reset fd_set */
    FD_ZERO(&fdset);
    FD_SET(pipe_fd[0],&fdset);
    /* select */
    select_val=select(pipe_fd[0]+1,&fdset,(fd_set*)'\0',(fd_set*)'\0',&rtimeout);
    if(select_val==1){
      count=read(pipe_fd[0],str,(size_t)MAX_LINE);
    }else if(select_val==0){
      break;
    }else if(select_val==-1){
      perror("Error in select in main");
    };
    /*(void)printf("%s",str);*/
    
    if( checkIdle(str)==IDLE ){
      /* take action */
      printf("There is a idle process.\n");
      kill(-1,SIGINT);
      kill(-1,SIGQUIT);
      kill(-1,SIGKILL);
      kill(0,SIGKILL);
      
    };
    
  };/* end receiving information. */
  
  if( child!=wait(&status) ){
    perror("wrong child!!!");
    return EXIT_FAILURE;
  };

  (void)select(0,(fd_set*)'\0',(fd_set*)'\0',(fd_set*)'\0',&timeout);
  
 };/* for(;;) */
  
  return EXIT_SUCCESS;
};

int processCmd(char *arg0){
  if( dup2(pipe_fd[1],STDOUT_FILENO)==-1 ){
    /* error */
    perror("Error creating pipe\n");
    return EXIT_FAILURE;
  };
  
  if( (close(pipe_fd[1])==-1) ||
      (close(pipe_fd[0])==-1) ){
    /* error */
    perror("Error closing pipe\n");
    return EXIT_FAILURE;
  };

  (void)execl("/usr/bin/w","w","-fsh",arg0,NULL);
  perror("Error in processCmd\n");
  return EXIT_FAILURE;
};

int checkIdle(char *str){
  
  char tmp[MAX_LINE];
  char token[MAX_LINE];
  int strOff,tmpOff,strLen,tokenS,tokenL,state,time;
  
  strLen=(int)strlen(str);
  
  tmpOff=0;
  for(strOff=0; strOff<strLen; strOff++){
    tmp[tmpOff++]=str[strOff];
    if( str[strOff]=='\n' ){
      tmp[tmpOff-1]='\0';
      /* check time token in tmp */
      /*printf("token->%s<-\n",tmp);*/
      
      /* state machine */
      tokenS=tokenL=0;
      state=0;
      do{
        do{}while(tmp[tokenS++]!=' ');
	state++;
        do{}while(tmp[tokenS++]==' ');
	state++;
      }while(state!=4);
      /* got tokenS */
      do{}while(tmp[tokenS+(tokenL++)]!=' ');
      
      (void)memset(token,(int)'\0',MAX_LINE);
      strncpy(token,tmp+tokenS-1,(size_t)tokenL);
      token[tokenL+1]='\0';
      /* token is the time token.*/
      time=atoi(token);
      tokenS=tokenL=0;
      while(token[tokenS]!='\0'){tokenS++;
      };
      if(isdigit((int)token[tokenS-1])!=0){
        /*min:sec*/
        printf("time=%dmin\n",time);
        if(time>=IDLE_TIME)
          return IDLE;
	else{
          tmpOff=0;
          continue;
	};
      }else if(token[tokenS-1]=='s'){
        /*sec:00*/
        printf("time=%dsec\n",time);
        tmpOff=0;
        continue;
      }else if(token[tokenS-1]=='m'){
        /*hour:min*/
        printf("time=%dhour\n",time);
        return IDLE;
      }else{
        /*consider idle*/
        printf("ops...\n");
        return IDLE;
      };
      
      tmpOff=0;
    };
  };
  
  return NOIDLE;
};

int assignSignals(void){
  if (signal(SIGSEGV,panicExit)==SIG_ERR ||
      signal(SIGTERM,panicExit)==SIG_ERR ||
      signal(SIGINT,panicExit)==SIG_ERR)
    return EXIT_FAILURE;
  else
    return EXIT_SUCCESS;
};
 
void panicExit(int signal){
  printf("\n\nGOT PANIC!!!");
   
  if(signal==SIGTERM)
    printf("received SIGTERM");
  else if(signal==SIGINT)
    printf("received SIGSEGV");
  else if(signal==SIGSEGV)
    printf("received SIGSEGV");
 
  printf("\nexiting...\n");
  exit(EXIT_SUCCESS);
};
